@@ -1,20 +1,6 @@
-/* Copyright (C) 2009-2022 Greenbone Networks GmbH
+/* SPDX-FileCopyrightText: 2023 Greenbone AG
  *
  * SPDX-License-Identifier: GPL-2.0-or-later
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -173,7 +159,7 @@ nasl_wmi_connect (lex_ctxt *lexic)
   argv[0] = g_strdup ("wmic");
   argv[1] = g_strdup ("-U");
   argv[2] = g_strdup_printf ("%s%%%s", username, password);
-  argv[3] = g_strdup_printf ("//%s%s", ip, options ? options : "");
+  argv[3] = g_strdup_printf ("//%s%s", ip, options ? options : "[sign]");
   argv[4] = g_strdup (ns);
   g_free (ip);
 
@@ -317,7 +303,7 @@ nasl_wmi_connect_rsop (lex_ctxt *lexic)
   argv[0] = g_strdup ("wmic");
   argv[1] = g_strdup ("-U");
   argv[2] = g_strdup_printf ("%s%%%s", username, password);
-  argv[3] = g_strdup_printf ("//%s%s", ip, options ? options : "");
+  argv[3] = g_strdup_printf ("//%s%s", ip, options ? options : "[sign]");
   g_free (ip);
 
   tree_cell *retc = alloc_typed_cell (CONST_INT);
@@ -430,7 +416,7 @@ nasl_wmi_connect_reg (lex_ctxt *lexic)
   argv[0] = g_strdup ("wmic");
   argv[1] = g_strdup ("-U");
   argv[2] = g_strdup_printf ("%s%%%s", username, password);
-  argv[3] = g_strdup_printf ("//%s%s", ip, options ? options : "");
+  argv[3] = g_strdup_printf ("//%s%s", ip, options ? options : "[sign]");
   g_free (ip);
 
   tree_cell *retc = alloc_typed_cell (CONST_INT);
