@@ -18,6 +18,7 @@ use std::{
 
 use hex::encode;
 use nasl_interpreter::{AsBufReader, LoadError};
+use nasl_syntax::Loader;
 use sha2::{Digest, Sha256};
 
 use openpgp::{
@@ -77,6 +78,7 @@ impl Display for Error {
         }
     }
 }
+impl std::error::Error for Error {}
 
 struct VHelper {
     keyring: String,
@@ -342,3 +344,88 @@ where
         self.next_filename()
     }
 }
+
+/// Finds .nasl and .inc files within a given path.
+///
+/// If the base is set it returns the relative path otherwise the absolute path.
+/// When the relative path is returned it behaves exactly like a `HashSumNameLoader`.
+pub struct NaslFileFinder {
+    base: Option<String>,
+    paths: glob::Paths,
+}
+
+impl NaslFileFinder {
+    /// Initializes NaslFileFinder based on a base path.
+    pub fn new<P>(base: P, relative: bool) -> Self
+    where
+        P: AsRef<str>,
+    {
+        let paths = glob::glob(&format!("{}/**/*", base.as_ref())).expect("valid glob pattern");
+        Self {
+            base: if relative {
+                Some(base.as_ref().to_string())
+            } else {
+                None
+            },
+            paths,
+        }
+    }
+}
+
+impl Loader for NaslFileFinder {
+    fn load(&self, key: &str) -> Result<String, LoadError> {
+        let path = if let Some(base) = &self.base {
+            let path: std::path::PathBuf = base.into();
+            path.join(key)
+        } else {
+            key.into()
+        };
+
+        // unfortunately nasl is still in iso-8859-1
+        nasl_syntax::load_non_utf8_path(path.as_path())
+    }
+
+    fn root_path(&self) -> Result<String, LoadError> {
+        self.base.clone().ok_or_else(|| {
+            LoadError::Dirty("NaslFileFinder is not initialized with a base path".to_string())
+        })
+    }
+}
+
+impl Iterator for NaslFileFinder {
+    type Item = Result<String, Error>;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        loop {
+            match self.paths.next() {
+                Some(result) => {
+                    let result = result.map_err(|e| {
+                        Error::LoadError(LoadError::Dirty(format!("Not a valid file: {}", e,)))
+                    });
+
+                    match result {
+                        Ok(f) => {
+                            let filename = f.display().to_string();
+                            if filename.ends_with(".nasl") | filename.ends_with(".inc") {
+                                let result = if let Some(base) = &self.base {
+                                    filename.trim_start_matches(base).trim_start_matches('/')
+                                } else {
+                                    &filename
+                                };
+                                return Some(Ok(result.to_owned()));
+                            }
+                        }
+                        Err(e) => return Some(Err(e)),
+                    }
+                }
+                None => return None,
+            }
+        }
+    }
+}
+
+impl FileNameLoader for NaslFileFinder {
+    fn next_filename(&mut self) -> Option<Result<String, Error>> {
+        self.next()
+    }
+}
