@@ -3,11 +3,12 @@
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 /// Information about hosts of a running scan
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Default, PartialEq, Eq)]
 #[cfg_attr(
     feature = "serde_support",
     derive(serde::Serialize, serde::Deserialize)
 )]
+#[cfg_attr(feature = "bincode_support", derive(bincode::Encode, bincode::Decode))]
 pub struct HostInfo {
     /// Number of all hosts, that are contained in a target
     pub all: u32,
@@ -23,8 +24,8 @@ pub struct HostInfo {
     pub finished: u32,
     #[cfg_attr(
         feature = "serde_support",
-        serde(skip_serializing_if = "Vec::is_empty")
+        serde(skip_serializing_if = "Option::is_none")
     )]
     /// IPs of hosts, that are currently scanned.
-    pub scanning: Vec<String>,
+    pub scanning: Option<Vec<String>>,
 }
