@@ -1,21 +1,8 @@
-/* Portions Copyright (C) 2009-2022 Greenbone Networks GmbH
- * Portions Copyright (C) 2006 Software in the Public Interest, Inc.
- * Based on work Copyright (C) 1998 - 2006 Tenable Network Security, Inc.
+/* SPDX-FileCopyrightText: 2023 Greenbone AG
+ * SPDX-FileCopyrightText: 2006 Software in the Public Interest, Inc.
+ * SPDX-FileCopyrightText: 1998-2006 Tenable Network Security, Inc.
  *
  * SPDX-License-Identifier: GPL-2.0-only
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * version 2 as published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /**
@@ -560,6 +547,12 @@ openvas (int argc, char *argv[], char *env[])
     }
   tzset ();
 
+#ifdef LOG_REFERENCES_AVAILABLE
+  if (scan_id)
+    set_log_reference (scan_id);
+  if (stop_scan_id)
+    set_log_reference (stop_scan_id);
+#endif // LOG_REFERENCES_AVAILABLE
   if (init_logging () != 0)
     return EXIT_FAILURE;
 
@@ -601,6 +594,9 @@ openvas (int argc, char *argv[], char *env[])
       set_scan_id (g_strdup (stop_scan_id));
       err = stop_single_task_scan ();
       gvm_close_sentry ();
+#ifdef LOG_REFERENCES_AVAILABLE
+      free_log_reference ();
+#endif // LOG_REFERENCES_AVAILABLE
       return err ? EXIT_FAILURE : EXIT_SUCCESS;
     }
 
@@ -621,6 +617,9 @@ openvas (int argc, char *argv[], char *env[])
 
       gvm_close_sentry ();
       destroy_scan_globals (globals);
+#ifdef LOG_REFERENCES_AVAILABLE
+      free_log_reference ();
+#endif // LOG_REFERENCES_AVAILABLE
       return EXIT_SUCCESS;
     }
 
